import {
  Controller,
  Get,
  Header,
  Param,
  ParseIntPipe,
  StreamableFile,
} from '@nestjs/common';
import { AppService } from './app.service';
import { ApiOperation, ApiParam } from '@nestjs/swagger';
import { BlankReturnMessageDto } from './dto/ReturnMessage.dto';

@Controller()
export class AppController {
  constructor(private readonly appService: AppService) {}

  @Get('user_avatar/ygobbs.com/:username/:size/:filename.png')
  @Header('Cache-Control', 'public, max-age=600')
  // @Header('Content-Type', 'image/png')
  @ApiOperation({ summary: '获取裁剪后的用户头像' })
  @ApiParam({ name: 'username', description: '用户名' })
  @ApiParam({ name: 'size', description: '头像尺寸', type: 'number' })
  @ApiParam({ name: 'filename', description: '头像文件名' })
  async getImage(
    @Param('username') username: string,
    @Param('size', ParseIntPipe) size: number,
  ) {
    if (size < 8 || size > 1000) {
      throw new BlankReturnMessageDto(400, 'Invalid size');
    }
    const buffer = await this.appService.getImage(username, size);
    return new StreamableFile(buffer, {
      type: 'image/png',
      disposition: 'inline',
    });
  }
}
